/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * vim: sw=2 ts=2 sts=2 expandtab 
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Shawn Wilsher <me@shawnwilsher.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

%{C++
#ifdef ERROR
#undef ERROR
#endif
%}

[scriptable, uuid(1f350f96-7023-434a-8864-40a1c493aac1)]
interface mozIStorageError : nsISupports {

  /**
   * General SQL error or missing database.
   */
  const long ERROR = 1;

  /**
   * Internal logic error.
   */
  const long INTERNAL = 2;

  /**
   * Access permission denied.
   */
  const long PERM = 3;

  /**
   * A callback routine requested an abort.
   */
  const long ABORT = 4;

  /**
   * The database file is locked.
   */
  const long BUSY = 5;

  /**
   * A table in the database is locked.
   */
  const long LOCKED = 6;

  /**
   * An allocation failed.
   */
  const long NOMEM = 7;

  /**
   * Attempt to write to a readonly database.
   */
  const long READONLY = 8;

  /**
   * Operation was terminated by an interrupt.
   */
  const long INTERRUPT = 9;

  /**
   * Some kind of disk I/O error occurred.
   */
  const long IOERR = 10;

  /**
   * The database disk image is malformed.
   */
  const long CORRUPT = 11;

  /**
   * An insertion failed because the database is full.
   */
  const long FULL = 13;

  /**
   * Unable to open the database file.
   */
  const long CANTOPEN = 14;

  /**
   * The database is empty.
   */
  const long EMPTY = 16;

  /**
   * The database scheme changed.
   */
  const long SCHEMA = 17;

  /**
   * A string or blob exceeds the size limit.
   */
  const long TOOBIG = 18;

  /**
   * Abort due to a constraint violation.
   */
  const long CONSTRAINT = 19;

  /**
   * Data type mismatch.
   */
  const long MISMATCH = 20;

  /**
   * Library used incorrectly.
   */
  const long MISUSE = 21;

  /**
   * Uses OS features not supported on the host system.
   */
  const long NOLFS = 22;

  /**
   * Authorization denied.
   */
  const long AUTH = 23;

  /**
   * Auxiliary database format error.
   */
  const long FORMAT = 24;

  /**
   * File opened that is not a database file.
   */
  const long NOTADB = 26;


  /**
   * Indicates what type of error occurred.
   */
  readonly attribute long result;

  /**
   * An error string the gives more details, if available.
   */
  readonly attribute AUTF8String message;
};
